@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the zim.fs module.'''
 
@@ -11,11 +11,27 @@ import tests
 import os
 import time
 
+import zim.fs
 from zim.fs import *
-from zim.fs import Path, FileHandle, FileWriteError, TmpFile, get_tmpdir, normalize_win32_share, PathLookupError, FileNotFoundError, FilteredDir, isabs, joinpath
 from zim.errors import Error
 
 
+def modify_file_mtime(path, func):
+	'''Helper function to modify a file in such a way that mtime
+	changed.
+	'''
+	mtime = os.stat(path).st_mtime
+	m = mtime
+	i = 0
+	while m == mtime:
+		time.sleep(1)
+		func(path)
+		m = os.stat(path).st_mtime
+		i += 1
+		assert i < 5
+	#~ print '>>>', m, mtime
+
+
 class FilterOverWriteWarning(tests.LoggingFilter):
 
 	logger = 'zim.fs'
@@ -43,28 +59,31 @@ class TestFS(tests.TestCase):
 				self.assertEqual(normalize_win32_share(url), url)
 
 		self.assertTrue(isabs('/foo/bar'))
+		self.assertTrue(isabs('~/foo/bar'))
 		self.assertFalse(isabs('./bar'))
 
 		self.assertEqual(joinpath('foo', 'bar'), os.sep.join(('foo', 'bar')))
 
-	def testPath(self):
+		self.assertEqual(cleanup_filename('foo&bar:dus\\foo.txt'), 'foo&bardusfoo.txt')
+
+	def testFilePath(self):
 		'''Test Path object'''
-		path = Path(['foo', 'bar'])
+		path = FilePath(['foo', 'bar'])
 		test = os.path.abspath( os.path.join('foo', 'bar') )
 		self.assertEqual(path.path, test)
 
-		path = Path('/foo/bar')
+		path = FilePath('/foo/bar')
 		uri = 'file:///' + os.path.abspath('/foo/bar').replace('\\', '/').strip('/')
 		self.assertEqual(path.uri, uri)
 
-		self.assertEqual(Path('file:///foo/bar'), Path('/foo/bar'))
-		self.assertEqual(Path('file:/foo/bar'), Path('/foo/bar'))
-		self.assertEqual(Path('file://localhost/foo/bar'), Path('/foo/bar'))
-		self.assertEqual(Path('file:///C:/foo/bar'), Path('/C:/foo/bar'))
+		self.assertEqual(FilePath('file:///foo/bar'), FilePath('/foo/bar'))
+		self.assertEqual(FilePath('file:/foo/bar'), FilePath('/foo/bar'))
+		self.assertEqual(FilePath('file://localhost/foo/bar'), FilePath('/foo/bar'))
+		self.assertEqual(FilePath('file:///C:/foo/bar'), FilePath('/C:/foo/bar'))
 		if os.name == 'nt':
-			self.assertEqual(Path('file:///C:/foo/bar'), Path(r'C:\foo\bar'))
+			self.assertEqual(FilePath('file:///C:/foo/bar'), FilePath(r'C:\foo\bar'))
 
-		path = Path('/foo//bar/baz/')
+		path = FilePath('/foo//bar/baz/')
 		drive, p = os.path.splitdrive(path.path)
 		self.assertEqual(path.split(), [drive + os.sep + 'foo', 'bar', 'baz'])
 		dirs = []
@@ -77,19 +96,19 @@ class TestFS(tests.TestCase):
 			('/foo/bar/baz/', '/foo/dus', '/foo'),
 			('/foo/bar', '/dus/ja', '/'),
 		):
-			self.assertEqual(Path(path1).commonparent(Path(path2)), Dir(common))
+			self.assertEqual(FilePath(path1).commonparent(FilePath(path2)), Dir(common))
 
 		if os.name == 'nt':
 			path1 = 'C:\foo\bar'
 			path2 = 'D:\foo\bar\baz'
-			self.assertEqual(Path(path1).commonparent(Path(path2)), None)
+			self.assertEqual(FilePath(path1).commonparent(FilePath(path2)), None)
 
 		for path1, path2, relpath in (
 			('/foo/bar/baz', '/foo', 'bar/baz'),
 		):
-			self.assertEqual(Path(path1).relpath(Path(path2)), relpath)
+			self.assertEqual(FilePath(path1).relpath(FilePath(path2)), relpath)
 
-		self.assertRaises(AssertionError, Path('/foo/bar').relpath, Path('/dus/ja'))
+		self.assertRaises(AssertionError, FilePath('/foo/bar').relpath, FilePath('/dus/ja'))
 
 		for path1, path2, relpath in (
 			('/foo/bar', '/dus/ja/', '../../foo/bar'),
@@ -97,28 +116,38 @@ class TestFS(tests.TestCase):
 			('/source/dir/foo/dus.pdf', '/source/dir/foo', 'dus.pdf'),
 			('/source/dir/dus.pdf', '/source/dir/foo', '../dus.pdf'),
 		):
-			self.assertEqual(Path(path1).relpath(Path(path2), allowupward=True), relpath)
+			self.assertEqual(FilePath(path1).relpath(FilePath(path2), allowupward=True), relpath)
 
 		if os.name == 'nt':
 			path1 = 'C:\foo\bar'
 			path2 = 'D:\foo\bar\baz'
-			self.assertEqual(Path(path1).relpath(Path(path2), allowupward=True), None)
+			self.assertEqual(FilePath(path1).relpath(FilePath(path2), allowupward=True), None)
 
-		self.assertEqual(Path('/foo') + 'bar', Path('/foo/bar'))
+		self.assertEqual(FilePath('/foo') + 'bar', FilePath('/foo/bar'))
+
+		path = FilePath('~/foo')
+		self.assertNotEqual(path.path, '~/foo')
+		self.assertEqual(path.user_path, '~/foo')
+		self.assertEqual(path.serialize_zim_config(), '~/foo')
+
+		path = FilePath('/foo')
+		self.assertIsNotNone(path.path)
+		self.assertIsNone(path.user_path)
+		self.assertIsNotNone(path.serialize_zim_config())
 
 		# Test unicode compat
 		string = u'\u0421\u0430\u0439\u0442\u043e\u0432\u044b\u0439'
-		path = Path(string)
+		path = FilePath(string)
 		self.assertTrue(path.path.endswith(string))
 		#~ self.assertRaises(Error, Path, string.encode('utf-8'))
-		path = Path((string, 'foo'))
+		path = FilePath((string, 'foo'))
 		self.assertTrue(path.path.endswith(os.sep.join((string, 'foo'))))
 		#~ self.assertRaises(Error, Path, (string.encode('utf-8'), 'foo'))
 
 	def testFileHandle(self):
 		'''Test FileHandle object'''
 		self.on_close_called = False
-		tmpdir = tests.create_tmp_dir('fs_testFile')
+		tmpdir = self.create_tmp_dir('testFileHandle')
 		fh = FileHandle(
 			tmpdir+'/foo.txt', mode='w', on_close=self.on_close)
 		fh.write('duss')
@@ -130,7 +159,7 @@ class TestFS(tests.TestCase):
 
 	def testFile(self):
 		'''Test File object'''
-		tmpdir = tests.create_tmp_dir('fs_testFile')
+		tmpdir = self.create_tmp_dir('testFile')
 		file = File(tmpdir+'/foo/bar/baz.txt')
 		assert not file.exists()
 		file.touch()
@@ -195,6 +224,10 @@ class TestFS(tests.TestCase):
 		file = File(tmpdir+'/newlines.txt')
 		self.assertEqual(file.read(), 'Some lines\nWith win32 newlines\n')
 
+		# test encoding error
+		non_utf8_file = File('tests/data/non-utf8.txt')
+		self.assertRaises(FileUnicodeError, non_utf8_file.read)
+
 		# test compare & copyto
 		file1 = File(tmpdir + '/foo.txt')
 		file2 = File(tmpdir + '/bar.txt')
@@ -230,7 +263,7 @@ class TestFS(tests.TestCase):
 
 	def testDir(self):
 		'''Test Dir object'''
-		tmpdir = tests.create_tmp_dir('fs_testDir')
+		tmpdir = self.create_tmp_dir('testDir')
 		dir = Dir(tmpdir+'/foo/bar')
 		assert not dir.exists()
 
@@ -261,12 +294,22 @@ class TestFS(tests.TestCase):
 
 		self.assertEqual(File((dir, 'foo.txt')), dir.file('foo.txt'))
 		self.assertEqual(dir.file(File((dir, 'foo.txt'))), dir.file('foo.txt'))
+		self.assertEqual(dir.file(FilePath((dir, 'foo.txt'))), dir.file('foo.txt'))
+		self.assertEqual(dir.file(('foo.txt',)), dir.file('foo.txt'))
 		self.assertRaises(PathLookupError, dir.file, File('/foo/bar.txt')) # not below dir
 
+		self.assertEqual(dir.resolve_file('../foo.txt'), dir.dir.file('foo.txt'))
+		self.assertEqual(dir.resolve_file(File('/foo/bar.txt')), File('/foo/bar.txt'))
+
 		self.assertEqual(Dir((dir, 'bar')), dir.subdir('bar'))
 		self.assertEqual(dir.subdir(Dir((dir, 'bar'))), dir.subdir('bar'))
+		self.assertEqual(dir.subdir(FilePath((dir, 'bar'))), dir.subdir('bar'))
+		self.assertEqual(dir.subdir(('bar',)), dir.subdir('bar'))
 		self.assertRaises(PathLookupError, dir.subdir, Dir('/foo/bar')) # not below dir
 
+		self.assertEqual(dir.resolve_dir('../bar'), dir.dir.subdir('bar'))
+		self.assertEqual(dir.resolve_dir(Dir('/foo/bar')), Dir('/foo/bar'))
+
 		self.assertRaises(OSError, dir.remove) # dir not empty
 		self.assertTrue(dir.exists())
 		dir.cleanup()
@@ -279,24 +322,14 @@ class TestFS(tests.TestCase):
 		self.assertEqual(dir.list(), []) # list non-existing dir
 
 
+@tests.slowTest
 class TestFileOverwrite(tests.TestCase):
 
-	slowTest = True
-
 	def setUp(self):
-		self.path = tests.create_tmp_dir('fs_testOverwrite')+'/file.txt'
+		self.path = self.create_tmp_dir()+'/file.txt'
 
 	def modify(self, func):
-		mtime = os.stat(self.path).st_mtime
-		m = mtime
-		i = 0
-		while m == mtime:
-			time.sleep(1)
-			func(self.path)
-			m = os.stat(self.path).st_mtime
-			i += 1
-			assert i < 5
-		#~ print '>>>', m, mtime
+		modify_file_mtime(self.path, func)
 
 	def runTest(self):
 		'''Test file overwrite check'''
@@ -329,22 +362,15 @@ class TestFileOverwrite(tests.TestCase):
 		self.assertEquals(file.read(), 'foo')
 
 
+@tests.slowTest
+@tests.skipUnless(hasattr(os, 'symlink'), 'OS does not supprot symlinks')
 class TestSymlinks(tests.TestCase):
 
-	slowTest = True
-
-	@staticmethod
-	def skipTest():
-		if not hasattr(os, 'symlink'):
-			return 'OS does not supprot symlinks'
-		else:
-			return False
-
 	def runTest(self):
 		'''Test file operations are safe for symlinks'''
 
 		# Set up a file structue with a symlink
-		tmpdir = tests.create_tmp_dir('fs_TestSymLinks')
+		tmpdir = self.create_tmp_dir()
 		targetdir = Dir(tmpdir + '/target')
 		targetdir.file('foo.txt').touch()
 		targetfile = File(tmpdir + '/target.txt')
@@ -383,3 +409,28 @@ class TestSymlinks(tests.TestCase):
 		self.assertEqual(dir.list(), [])
 		self.assertTrue(targetdir.exists())
 		self.assertEqual(targetdir.list(), ['foo.txt'])
+
+
+@tests.slowTest
+@tests.skipUnless(zim.fs.gio, 'Trashing not supported, \'gio\' is missing')
+class TestTrash(tests.TestCase):
+
+	def runTest(self):
+		'''Test trashing files and folders'''
+		root = Dir(self.create_tmp_dir())
+		file = root.file('test.txt')
+		file.touch()
+		self.assertTrue(file.exists())
+		self.assertTrue(file.trash())
+		self.assertFalse(file.exists())
+		dir = root.subdir('test')
+		dir.touch()
+		self.assertTrue(dir.exists())
+		self.assertTrue(dir.trash())
+		self.assertFalse(dir.exists())
+
+		# fails silent if file does not exist
+		self.assertFalse(file.trash())
+		self.assertFalse(dir.trash())
+
+		# How can we cause gio to give an error and test that case ??
