@@ -2,7 +2,7 @@
 
 # Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from tests import TestCase, get_test_notebook
+import tests
 
 import os
 
@@ -14,7 +14,7 @@ assert len(zim.plugins.__path__) > 1 # test __path__ magic
 zim.plugins.__path__ = [os.path.abspath('./zim/plugins')] # set back default search path
 
 
-class testPlugins(TestCase):
+class testPlugins(tests.TestCase):
 	'''FIXME'''
 
 	def testListAll(self):
@@ -35,20 +35,23 @@ class testPlugins(TestCase):
 
 			# test dependencies data
 			dep = plugin.check_dependencies()
+			self.assertTrue(isinstance(dep,tuple))
+			check, dep = dep
+			self.assertTrue(isinstance(check,bool))
 			self.assertTrue(isinstance(dep,list))
 			for i in range(len(dep)):
 				self.assertTrue(isinstance(dep[i],tuple))
 				self.assertTrue(isinstance(dep[i][0],str))
 				self.assertTrue(isinstance(dep[i][1],bool))
+				self.assertTrue(isinstance(dep[i][2],bool))
 
 	def testDefaulPlugins(self):
 		'''Test loading default plugins'''
 		# Note that we use parent interface class here, so plugins
 		# will not really attach - just testing loading and prereq
 		# checks are OK.
-		notebook = get_test_notebook()
+		notebook = tests.new_notebook()
 		interface = zim.NotebookInterface(notebook)
 		interface.uistate = zim.config.ConfigDict()
 		interface.load_plugins()
 		self.assertTrue(len(interface.plugins) > 3)
-
