@@ -2,16 +2,17 @@
 
 # Copyright 2011 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from tests import TestCase, MockObject, get_test_notebook
+import tests
+
 
 import zim.plugins
 import zim.config
 import zim.formats
 
 
-class TestTaskList(TestCase):
+class TestTaskList(tests.TestCase):
 
-	def runTest(self):
+	def testIndexing(self):
 		'''Check indexing of tasklist plugin'''
 		klass = zim.plugins.get_plugin('tasklist')
 		ui = MockUI()
@@ -49,24 +50,32 @@ FIXME: dus
 		tasks = plugin.extract_tasks(tree)
 		labels = [task[-1] for task in tasks]
 		self.assertEqual(labels, ['A', 'B', 'C', 'D', 'E', 'FIXME: dus'])
-		self.assertEqualDiff(tree.tostring(), origtree)
+		self.assertEqual(tree.tostring(), origtree)
 			# extract should not modify the tree
 
 		plugin.preferences['all_checkboxes'] = False
 		tasks = plugin.extract_tasks(tree)
 		labels = [task[-1] for task in tasks]
 		self.assertEqual(labels, ['A', 'B', 'C', 'FIXME: dus'])
-		self.assertEqualDiff(tree.tostring(), origtree)
+		self.assertEqual(tree.tostring(), origtree)
 			# extract should not modify the tree
 
 		# TODO: test tags, due dates, tags for whole list, etc.
 
+	def testDialog(self):
+		'''Check tasklist plugin dialog'''
+		klass = zim.plugins.get_plugin('tasklist')
+		ui = MockUI()
+		plugin = klass(ui)
+		ui.notebook.index.flush()
+		ui.notebook.index.update()
 
 
-class MockUI(MockObject):
+
+class MockUI(tests.MockObject):
 
 	def __init__(self):
-		MockObject.__init__(self)
+		tests.MockObject.__init__(self)
 		self.preferences = zim.config.ConfigDict()
 		self.uistate = zim.config.ConfigDict()
-		self.notebook = get_test_notebook()
+		self.notebook = tests.new_notebook()
