@@ -2,8 +2,89 @@
 
 # Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
+'''
+This module contains the base class for the zim application and the main
+function. The rest of the implementation is divided over it's sub-modules.
+
+B{NOTE:} There is also some generic development documentation in the
+"HACKING" folder in the source distribution. Please also have a look
+at that if you want to help with zim development.
+
+In this API documentation many of the methods with names starting with
+C{do_} and C{on_} are not documented. The reason is that these are
+signal handlers that are not part of the external API. They act upon
+a signal but should never be called directly by other objects.
+
+
+Overview
+========
+
+The script C{zim.py} is a thin wrapper around the L{main()} function
+defined here. THe main function validates commandline options and if
+all is well it either calls the background daemon to connect to some
+running instance of zim, or it instantiates a L{NotebookInterface}
+object, or an object of a subclass like L{GtkInterface} (for the
+graphic user interface) or L{WWWInterface} (for the webinterface).
+
+The L{NotebookInterface} class takes care of connecting to a L{Notebook}
+object and help with e.g. loading plugins and config files. It's
+subclasses build on top of this to implement specific user interfaces.
+The graphical user interface is implemented in the L{zim.gui} module
+and it's sub-modules. The webinterface is implemented in L{zim.www}.
+
+The graphical interface uses a background process to coordinate
+between instances, this is implemented in L{zim.daemon}.
+
+Regardsless of the interface choosen there is a L{Notebook} object
+which implements a generic API for accessing and storing pages and
+other data in the notebook. The notebook object is agnostic about the
+actual source of the data (files, database, etc.), this is implemented
+by "store" objects which handle a specific storage model. Storage models
+live below the L{zim.stores} module; e.g. the default mapping of a
+notebook to a folder with one file per page is implemented in the module
+L{zim.stores.files}.
+
+The notebook works together with an L{Index} object which keeps a
+database of all the pages to speed up notebook access and allows us
+to e.g. show a list of pages in the side pane of the user interface.
+
+Another aspect of the notebook is the parsing of the wiki text in the
+pages and contruct a tree model of the formatting that can be shown
+in the interface or exported to another format like HTML. There are
+several parsers which live below L{zim.formats}. The exporting is done
+by L{zim.exporter} and L{zim.templates} implements the template
+engine.
+
+Many classes in zim have signals which allow other objects to connect
+to a listen for specific events. This allows for an event driven chain
+of control, which is mainly used in the graphical interface. If you are
+not familiar with event driven programs please refer to a Gtk manual.
+
+
+Infrastructure classes
+======================
+
+All functions and objects to interact with the file system can be
+found in L{zim.fs}. For all functionality related to config files
+see L{zim.config}. For executing external applications see
+L{zim.applications} or L{zim.gui.applications}.
+
+For asynchronous actions see L{zim.async}.
+
+
+
+@newfield signal: Signal, Signals
+@newfield emits: Emits, Emits
+@newfield implementation: Implementation
+'''
+# New epydoc fields defined above are inteded as follows:
+# @signal: signal-name (param1, param2): description
+# @emits: signal
+# @implementation: must implement / optional for sub-classes
+
+
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.52'
+__version__ = '0.53'
 __url__='http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
 __copyright__ = 'Copyright 2008 - 2011 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
@@ -26,8 +107,7 @@ import logging
 
 from getopt import gnu_getopt, GetoptError
 
-import zim.fs
-from zim.fs import *
+from zim.fs import File, Dir
 from zim.errors import Error
 from zim.config import data_dir, config_file, log_basedirs, ZIM_DATA_DIR
 
@@ -44,16 +124,25 @@ else:
 
 gettext.install('zim', localedir, unicode=True, names=('_', 'gettext', 'ngettext'))
 
+
+#: This parameter can be set by ./setup.py, can be e.g. "maemo"
+PLATFORM = None
+
+#: Executable for starting new zim instances, set by main()
 ZIM_EXECUTABLE = 'zim'
 
 
 # All commandline options in various groups
 longopts = ('verbose', 'debug')
-commands = ('help', 'version', 'gui', 'server', 'export', 'index', 'manual', 'plugin', 'daemon')
+commands = (
+	'help', 'version', 'gui', 'server', 'export', 'search',
+	'index', 'manual', 'plugin', 'daemon'
+)
 commandopts = {
 	'gui': ('list', 'geometry=', 'fullscreen', 'no-daemon'),
 	'server': ('port=', 'template=', 'gui', 'no-daemon'),
 	'export': ('format=', 'template=', 'output=', 'root-url=', 'index-page='),
+	'search': (),
 	'index': ('output=',),
 	'plugin': (),
 	'daemon': (),
@@ -65,15 +154,16 @@ shortopts = {
 }
 maxargs = {
 	'gui': 2, 'server': 1, 'manual': 1,
-	'export': 2, 'index': 1
+	'export': 2, 'index': 1, 'search': 2,
 }
 
 # Inline help - do not use __doc__ for this !
 usagehelp = '''\
 usage: zim [OPTIONS] [NOTEBOOK [PAGE]]
-   or: zim --export [OPTIONS] NOTEBOOK [PAGE]
-   or: zim --index  [OPTIONS] NOTEBOOK
    or: zim --server [OPTIONS] [NOTEBOOK]
+   or: zim --export [OPTIONS] NOTEBOOK [PAGE]
+   or: zim --search NOTEBOOK QUERY
+   or: zim --index  [OPTIONS] NOTEBOOK
    or: zim --plugin PLUGIN [ARGUMENTS]
    or: zim --manual [OPTIONS] [PAGE]
    or: zim --help
@@ -113,6 +203,9 @@ Export Options:
   You can use the export option to print a single page to stdout.
   When exporting a whole notebook you need to provide a directory.
 
+Search Options:
+  None
+
 Index Options:
   -o, --output    output file
 
@@ -121,10 +214,14 @@ Try 'zim --manual' for more help.
 
 
 class UsageError(Error):
-	pass
+	'''Error when commandline usage is not correct'''
+
+	def __init__(self):
+		self.msg = usagehelp.replace('zim', ZIM_EXECUTABLE)
 
 
 class NotebookLookupError(Error):
+	'''Error when failing to locate a notebook'''
 
 	description = _('Could not find the file or folder for this notebook')
 		# T: Error verbose description
@@ -143,7 +240,17 @@ def _get_default_or_only_notebook():
 
 
 def main(argv):
-	'''Run the main program.'''
+	'''Run the main program
+
+	Depending on the commandline given and whether or not there is
+	an instance of zim running already, this method may return
+	immediatly, or go into the mainloop untill the program is exitted.
+
+	@param argv: commandline arguments, e.g. from C{sys.argv}
+
+	@raises UsageError: when number of arguments is not correct
+	@raises GetOptError: when invalid options are found
+	'''
 	global ZIM_EXECUTABLE
 
 	# FIXME - this returns python.exe on my windows test
@@ -240,7 +347,7 @@ def main(argv):
 			logger.debug(
 				'Zim revision is:\n'
 				'\tbranch: %(branch_nick)s\n'
-				'\trevision: %(revno)d %(revision_id)s\n'
+				'\trevision: %(revno)s %(revision_id)s\n'
 				'\tdate: %(date)s\n',
 				version_info )
 		except ImportError:
@@ -251,16 +358,21 @@ def main(argv):
 	# Now we determine the class to handle this command
 	# and start the application ...
 	logger.debug('Running command: %s', cmd)
-	if cmd in ('export', 'index'):
+	if cmd in ('export', 'index', 'search'):
 		if not len(args) >= 1:
 			default = _get_default_or_only_notebook()
+			if not default:
+				raise UsageError
 			handler = NotebookInterface(notebook=default)
 		else:
 			handler = NotebookInterface(notebook=args[0])
 
 		handler.load_plugins() # should this go somewhere else ?
 
-		if len(args) == 2:
+		if cmd == 'search':
+			if not len(args) == 2: raise UsageError
+			optsdict['query'] = args[1]
+		elif len(args) == 2:
 			optsdict['page'] = args[1]
 
 		method = getattr(handler, 'cmd_' + cmd)
@@ -297,7 +409,7 @@ def main(argv):
 				import zim.gui.notebookdialog
 				notebook = zim.gui.notebookdialog.prompt_notebook()
 				if not notebook:
-					return # User cancelled notebook dialog
+					return # User canceled notebook dialog
 			handler = zim.gui.GtkInterface(notebook, page, **optsdict)
 			handler.main()
 		else:
@@ -311,7 +423,7 @@ def main(argv):
 				notebook = zim.gui.notebookdialog.prompt_notebook()
 				if not notebook:
 					proxy.quit_if_nochild()
-					return # User cancelled notebook dialog
+					return # User canceled notebook dialog
 			gui = proxy.get_notebook(notebook)
 
 			gui.present(page, **optsdict)
@@ -340,16 +452,33 @@ def main(argv):
 
 
 class NotebookInterface(gobject.GObject):
-	'''Application wrapper for a notebook. Base class for GtkInterface
-	and WWWInterface classes.
+	'''Base class for the application object
 
-	Subclasses can prove a class attribute "ui_type" to tell plugins what
-	interface they support. This can be "gtk" or "html". If "ui_type" is None
-	we run without interface (e.g. commandline export).
+	This is the base class for application classes like L{GtkInterface}
+	and L{WWWInterface}. It can also be instantiated on it's own, which
+	should only be done for running commandline commands like export
+	and index.
 
-	Signals:
-	* open-notebook (notebook)
-	  Emitted to open a notebook in this interface
+	In the current design an application object can only open one
+	notebook. Also it is not possible to close the notebook and open
+	another one in the same interface. In practise this means that
+	each notebook that is opened runs in it's own process with it's
+	own application object.
+
+	@signal: C{open-notebook (notebook)}:
+	Emitted to open a notebook in this interface
+
+	@cvar ui_type: string to tell plugins what interface is supported
+	by this class. Currently this can be "gtk" or "html". If "ui_type"
+	is None we run without interface (e.g. commandline export).
+
+	@ivar notebook: the L{Notebook} that is open in this interface
+	@ivar plugins: list of L{plugin<zim.plugins>} objects that are
+	active
+	@ivar preferences: a L{ConfigDict} for the user preferences
+	(the C{X{preferences.conf}} config file)
+	@ivar uistate:  L{ConfigDict} with notebook specific interface state
+	(the C{X{state.conf}} file in the notebook cache folder)
 	'''
 
 	# define signals we want to use - (closure type, return type and arg types)
@@ -360,6 +489,12 @@ class NotebookInterface(gobject.GObject):
 	ui_type = None
 
 	def __init__(self, notebook=None):
+		'''Constructor
+
+		@keyword notebook: the L{Notebook} object to open in this
+		interface. If not specified here you can call L{open_notebook()}
+		to open one later.
+		'''
 		gobject.GObject.__init__(self)
 		self.notebook = None
 		self.plugins = []
@@ -371,13 +506,17 @@ class NotebookInterface(gobject.GObject):
 			self.open_notebook(notebook)
 
 	def load_plugins(self):
-		'''Load the plugins defined in the preferences'''
+		'''Loads all the plugins defined in the preferences
+
+		Typically called from the constructor of sub-classes.
+		'''
 		default = ['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol']
 		self.preferences['General'].setdefault('plugins', default)
 		plugins = self.preferences['General']['plugins']
 		plugins = set(plugins) # Eliminate doubles
+
 		# Plugins should not have dependency on order of being added
-		# just add sort to make behavior predictable.
+		# but sort them here to make behavior predictable.
 		for name in sorted(plugins):
 			self.load_plugin(name)
 
@@ -386,9 +525,29 @@ class NotebookInterface(gobject.GObject):
 			self.preferences['General']['plugins'] = sorted(loaded)
 
 	def load_plugin(self, name):
-		'''Load a single plugin by name, returns boolean for success'''
+		'''Load a single plugin by name
+
+		Load an plugin object and attach it to the current application
+		object. And add it to the preferences.
+
+		When the plugin was loaded already the already active object
+		will be returned. Thus for each plugin only one instance can be
+		active.
+
+		@param name: the plugin name as understood by
+		L{zim.plugins.get_plugin()}
+
+		@returns: the plugin object or C{None} when failed
+
+		@todo: make load_plugin raise exception on failure
+		'''
 		assert isinstance(name, basestring)
 		import zim.plugins
+
+		loaded = [p.plugin_key for p in self.plugins]
+		if name in loaded:
+			return self.plugins[loaded.index(name)]
+
 		try:
 			klass = zim.plugins.get_plugin(name)
 			if not klass.check_dependencies_ok():
@@ -408,10 +567,16 @@ class NotebookInterface(gobject.GObject):
 		return plugin
 
 	def unload_plugin(self, plugin):
-		'''Remove a plugin'''
+		'''Remove a plugin
+
+		De-attached the plugin from to the current application
+		object. And remove it from the preferences.
+
+		@param plugin: a plugin name or plugin object
+		'''
 		if isinstance(plugin, basestring):
 			name = plugin
-			assert name in map(lambda p: p.plugin_key, self.plugins)
+			assert name in [p.plugin_key for p in self.plugins]
 			plugin = filter(lambda p: p.plugin_key == name, self.plugins)[0]
 		else:
 			assert plugin in self.plugins
@@ -425,12 +590,28 @@ class NotebookInterface(gobject.GObject):
 		self.preferences.write()
 
 	def open_notebook(self, notebook):
-		'''Open a notebook if no notebook was set already.
-		'notebook' can be either a string, a File or Dir object or a
-		Notebook object.
+		'''Open the notebook object
 
-		If the notebook is a string which also specifies a page the page
-		path is returned so it can be handled in a sub-class.
+		Open the notebook object for this interface if no notebook was
+		set already.
+
+		@param notebook: either a string, a L{File} or L{Dir} object,
+		or a L{Notebook} object.
+
+		When the notebook is not given as a Notebook object,
+		L{zim.notebook.resolve_notebook()} is used to resolve it.
+		If this method returns a page as well it is returned here
+		so it can be handled in a sub-class.
+
+		The reason that we call C{resolve_notebook()} from here (instead
+		of resolving it first and than calling this method only with a
+		notebook object) is that we want to allow active plugins to
+		handle the notebook uri before loading the Notebook object
+		(e.g. to auto-mount the notebook folder).
+
+		@emits: open-notebook
+
+		@returns: a L{Path} if any was specified in the notebook spec
 		'''
 		from zim.notebook import resolve_notebook, get_notebook, Notebook
 		assert self.notebook is None, 'BUG: other notebook opened already'
@@ -468,10 +649,21 @@ class NotebookInterface(gobject.GObject):
 			from zim.config import ConfigDictFile
 			self.uistate = ConfigDictFile(
 				notebook.cache_dir.file('state.conf') )
-		# TODO read profile preferences file if one is set in the notebook
+		else:
+			from zim.config import ConfigDict
+			self.uistate = ConfigDict()
 
 	def cmd_export(self, format='html', template=None, page=None, output=None, root_url=None, index_page=None):
-		'''Method called when doing a commandline export'''
+		'''Convenience method hat wraps L{zim.exporter.Exporter} for
+		commandline export
+
+		@keyword format: the format name
+		@keyword template: the template path or name
+		@keyword page: the page name or C{None} to export the full notebook
+		@keyword output: the output folder or C{None} to print to stdout
+		@keyword root_url: the url to map the document root if any
+		@keyword index_page: the index page name if any
+		'''
 		import zim.exporter
 		exporter = zim.exporter.Exporter(self.notebook, format, template, document_root_url=root_url, index_page=index_page)
 
@@ -480,7 +672,6 @@ class NotebookInterface(gobject.GObject):
 			page = self.notebook.get_page(path)
 
 		if page and output is None:
-			import sys
 			exporter.export_page_to_fh(sys.stdout, page)
 		elif not output:
 			logger.error('Need output directory to export notebook')
@@ -492,8 +683,23 @@ class NotebookInterface(gobject.GObject):
 				self.notebook.index.update()
 				exporter.export_all(dir)
 
+	def cmd_search(self, query):
+		from zim.search import SearchSelection, Query
+		query = query.strip()
+		if not query: raise AssertionError, 'Empty query'
+		logger.info('Searching for: %s', query)
+		selection = SearchSelection(self.notebook)
+		query = Query(query)
+		selection.search(query)
+		for path in sorted(selection, key=lambda p: p.name):
+			print path.name
+
 	def cmd_index(self, output=None):
-		'''Method called when doing a commandline index re-build'''
+		'''Convenience method for the commandline 'index' command
+
+		@keyword output: the index file to update, defaults to the
+		default index s used by the notebook
+		'''
 		if not output is None:
 			import zim.index
 			index = zim.index.Index(self.notebook, output)
@@ -506,13 +712,17 @@ class NotebookInterface(gobject.GObject):
 		index.update(callback=on_callback)
 
 	def spawn(self, *args):
-		'''Spawn a new instance of zim'''
-		# TODO: after implementing the daemon, put this in that module
+		'''Spawn a new instance of zim
+
+		Run a new zim process with commandline ares
+
+		@param args: any commandline args to pass to the new process
+
+		@todo: take this method outside this class
+		'''
 		from zim.applications import Application
 		zim = Application((ZIM_EXECUTABLE,) + args)
 		zim.spawn()
 
 # Need to register classes defining gobject signals
 gobject.type_register(NotebookInterface)
-
-
