@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module supports dumping to HTML'''
 
@@ -111,7 +111,11 @@ class Dumper(DumperClass):
 				for o in ('width', 'height'):
 					if o in element.attrib and int(float(element.attrib[o])) > 0:
 						opt = ' %s="%s"' % (o, element.attrib[o])
-				output.append('<img src="%s" alt="%s"%s>' % (src, text, opt))
+				if 'href' in element.attrib:
+					href = self.linker.link(element.attrib['href'])
+					output.append('<a href="%s"><img src="%s" alt="%s"%s></a>' % (href, src, text, opt))
+				else:
+					output.append('<img src="%s" alt="%s"%s>' % (src, text, opt))
 			elif element.tag == 'link':
 				href = self.linker.link(element.attrib['href'])
 				title = text.replace('"', '&quot;')
@@ -121,6 +125,8 @@ class Dumper(DumperClass):
 				elif element.tag == 'emphasis': tag = 'em'
 				else: tag = element.tag
 				output += ['<', tag, '>', text, '</', tag, '>']
+			elif element.tag == 'tag':
+				output += ['<span class="zim-tag">', text, '</span>']
 			else:
 				assert False, 'Unknown node type: %s' % element
 
@@ -131,5 +137,3 @@ class Dumper(DumperClass):
 					# for whitespace between headings, paras etc.
 					tail = encode_whitespace(tail)
 				output.append(tail)
-
-
