@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Utilities to work with the clipboard for copy-pasting
 
@@ -10,14 +10,13 @@ straight forward API.
 '''
 
 # TODO support converting HTML to parsetree - need html Parser
-# TODO support for pasting image as parsetree - attach + tree ?
-# TODO unit test for copy - paste parsetree & page link
 
 import gtk
 import logging
 
 from zim.fs import File
-from zim.parsing import is_url_re, url_encode, URL_ENCODE_READABLE
+from zim.notebook import Path
+from zim.parsing import is_url_re, url_encode, link_type, URL_ENCODE_READABLE
 from zim.formats import get_format, ParseTree, TreeBuilder
 from zim.exporter import StaticLinker
 
@@ -26,7 +25,7 @@ logger = logging.getLogger('zim.gui.clipboard')
 
 
 # Targets have format (name, flags, id), flags need be 0 to allow
-# paste to other widgets and to other aplication instances
+# paste to other widgets and to other application instances
 PARSETREE_TARGET_ID = 1
 PARSETREE_TARGET_NAME = 'text/x-zim-parsetree'
 PARSETREE_TARGET = (PARSETREE_TARGET_NAME, 0, PARSETREE_TARGET_ID)
@@ -46,6 +45,16 @@ IMAGE_TARGETS = tuple(gtk.target_list_add_image_targets(info=IMAGE_TARGET_ID))
 	# but seems docs are not correct
 IMAGE_TARGET_NAMES = tuple([target[0] for target in IMAGE_TARGETS])
 
+# Add image format names as well, seen these being used by MS Office
+for format in gtk.gdk.pixbuf_get_formats():
+	if format['mime_types'][0] in IMAGE_TARGET_NAMES:
+		for n in (format['name'], format['name'].upper()):
+			IMAGE_TARGET_NAMES += (n,)
+			IMAGE_TARGETS += ((n, 0, IMAGE_TARGET_ID),)
+
+#~ print IMAGE_TARGETS
+#~ print IMAGE_TARGET_NAMES
+
 URI_TARGET_ID = 7
 URI_TARGETS = tuple(gtk.target_list_add_uri_targets(info=URI_TARGET_ID))
 	# According to docs we should provide list as arg to this function,
@@ -73,8 +82,8 @@ PARSETREE_ACCEPT_TARGET_NAMES = tuple([target[0] for target in PARSETREE_ACCEPT_
 
 
 # Mimetype text/uri-list is used for drag n drop of URLs
-# it is plain text encoded list of urls, seperated by \r\n
-# Since not all apps foolow the standard exactly, do allow for
+# it is plain text encoded list of urls, separated by \r\n
+# Since not all apps follow the standard exactly, do allow for
 # just \n separated lists.
 #
 # Zim page links are also encoded in this format, but no url encoding
@@ -97,35 +106,52 @@ def unpack_urilist(text):
 	return [line.decode('utf-8') for line in lines]
 
 
-def _file_link_tree(links, notebook, page):
-		#~ print 'LINKS: ', links
-		builder = TreeBuilder()
-		builder.start('zim-tree')
-		for i in range(len(links)):
-			if i > 0:
-				builder.data(' ')
+def _link_tree(links, notebook, page):
+	# Convert a list of links (of any type) into a parsetree
+	#~ print 'LINKS: ', links
+	#~ print 'NOTEBOOK and PAGE:', notebook, page
+	builder = TreeBuilder()
+	builder.start('zim-tree')
+	for i in range(len(links)):
+		if i > 0:
+			builder.data(' ')
 
-			isimage = False
-			if links[i].startswith('file:/'):
-				try:
-					isimage = File(links[i]).isimage()
-				except:
-					pass
+		link = links[i]
+		type = link_type(links[i])
+		isimage = False
+		if type == 'file':
+			try:
+				file = File(link)
+				isimage = file.isimage()
+			except:
+				pass
 
-			if isimage:
-				file = File(links[i])
-				src = notebook.relative_filepath(file, page) or file.path
-				builder.start('img', {'src': src})
-				builder.end('img')
-			else:
-				builder.start('link', {'href': links[i]})
-				builder.data(links[i])
-				builder.end('link')
-		builder.end('zim-tree')
-		tree = ParseTree(builder.close())
-		tree.resolve_images(notebook, page)
-		tree.decode_urls()
-		return tree
+		if isimage:
+			src = notebook.relative_filepath(file, page) or file.uri
+			builder.start('img', {'src': src})
+			builder.end('img')
+		elif link.startswith('@'):
+			# FIXME - is this ever used ??
+			builder.start('tag', {'name': links[i][1:]})
+			builder.data(links[i])
+			builder.end('tag')
+		else:
+			if type == 'page':
+				href = Path(notebook.cleanup_pathname(link)) # Assume links are always absolute
+				link = notebook.relative_link(page, href) or link
+			elif type == 'file':
+				file = File(link) # Assume links are always URIs
+				link = notebook.relative_filepath(file, page) or file.uri
+
+			builder.start('link', {'href': link})
+			builder.data(link)
+			builder.end('link')
+
+	builder.end('zim-tree')
+	tree = ParseTree(builder.close())
+	tree.resolve_images(notebook, page)
+	tree.decode_urls()
+	return tree
 
 
 def _get_image_info(targetname):
@@ -133,13 +159,13 @@ def _get_image_info(targetname):
 	# in available pixbuf writers for this type and return the
 	# format name and file extension
 	for format in gtk.gdk.pixbuf_get_formats():
-		if not targetname in format['mime_types']:
-			continue
-
-		if format['is_writable']:
-			return format['name'], format['extensions'][0]
-		else:
-			return None, None
+		if targetname == format['name'] \
+		or targetname == format['name'].upper() \
+		or targetname in format['mime_types']:
+			if format['is_writable']:
+				return format['name'], format['extensions'][0]
+			else:
+				return None, None
 	else:
 		return None, None
 
@@ -163,7 +189,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, page=None):
 	elif targetname in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) \
 	or targetname in URI_TARGET_NAMES:
 		links = unpack_urilist(selectiondata.data)
-		return _file_link_tree(links, notebook, page)
+		return _link_tree(links, notebook, page)
 	elif targetname in TEXT_TARGET_NAMES:
 		# plain text parser should highlight urls etc.
 		# FIXME some apps drop text/uri-list as a text/plain mimetype
@@ -190,7 +216,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, page=None):
 		pixbuf.save(file.path, format)
 
 		links = [file.uri]
-		return _file_link_tree(links, notebook, page)
+		return _link_tree(links, notebook, page)
 	else:
 		return None
 
@@ -202,7 +228,7 @@ HTML_HEAD = '''\
 '''.strip()
 
 def wrap_html(html, target):
-	'''Fucntion to wrap html with appropriate headers based on target type'''
+	'''Function to wrap html with appropriate headers based on target type'''
 	html = html.encode('utf-8')
 	if target == 'HTML Format':
 		return Win32HtmlFormat.encode(html, head=HTML_HEAD)
@@ -226,10 +252,15 @@ class Clipboard(gtk.Clipboard):
 	zim pageview widget.
 	'''
 
-	def set_parsetree(self, notebook, page, parsetree):
+	def set_parsetree(self, notebook, page, parsetree, format='plain'):
 		'''Copy a parsetree to the clipboard. The parsetree can be pasted by
 		the user either as formatted text within zim or as plain text outside
 		zim. The tree can be the full tree for 'page', but also a selection.
+
+		@param notebook: the L{Notebook} object
+		@param page: the L{Page} object - used to resolve links etc.
+		@param parsetree: the actual L{ParseTree} to be set on the clipboard
+		@keyword format: the format to use for pasting text, 'wiki' or 'plain'
 		'''
 		targets = [PARSETREE_TARGET]
 		targets.extend(HTML_TARGETS)
@@ -237,12 +268,12 @@ class Clipboard(gtk.Clipboard):
 		self.set_with_data(
 			targets,
 			Clipboard._get_parsetree_data, Clipboard._clear_data,
-			(notebook, page, parsetree)
+			(notebook, page, parsetree, format)
 		) or logger.warn('Failed to set data on clipboard')
 
 	def _get_parsetree_data(self, selectiondata, id, data):
-		logger.debug("Cliboard data request of type '%s', we have a parsetree", selectiondata.target)
-		notebook, page, parsetree = data
+		logger.debug("Clipboard data request of type '%s', we have a parsetree", selectiondata.target)
+		notebook, page, parsetree, format = data
 		if id == PARSETREE_TARGET_ID:
 			xml = parsetree.tostring().encode('utf-8')
 			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
@@ -263,18 +294,9 @@ class Clipboard(gtk.Clipboard):
 			#~ print 'PASTING: >>>%s<<<' % html
 			selectiondata.set(selectiondata.target, 8, html)
 		elif id == TEXT_TARGET_ID:
-			# FIXME no using wiki although plain is preferred
-			# because plain does not yet support lists etc.
-			dumper = get_format('wiki').Dumper()
+			dumper = get_format(format).Dumper()
 			text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
 			selectiondata.set_text(text)
-
-			# Pasting as 'plain' text because wiki syntax more often than not looks
-			# strange in external applications. Would need extra menu item to
-			# "copy as wiki text".
-			#dumper = get_format('plain').Dumper()
-			#text = ''.join( dumper.dump(parsetree) ).encode('utf-8')
-			#selectiondata.set_text(text)
 		else:
 			assert False, 'Unknown target id %i' % id
 
@@ -329,7 +351,7 @@ class Clipboard(gtk.Clipboard):
 		) or logger.warn('Failed to set data on clipboard')
 
 	def _get_pagelink_data(self, selectiondata, id, data):
-		logger.debug("Cliboard data request of type '%s', we have pagelink", selectiondata.target)
+		logger.debug("Clipboard data request of type '%s', we have pagelink", selectiondata.target)
 		notebookname, pagename = data
 		if id == INTERNAL_PAGELIST_TARGET_ID:
 			text = pack_urilist((pagename,))
@@ -344,7 +366,7 @@ class Clipboard(gtk.Clipboard):
 			assert False, 'Unknown target id %i' % id
 
 	def _clear_data(self, data):
-		# Callback to clear our cliboard data - pass because we keep no state
+		# Callback to clear our clipboard data - pass because we keep no state
 		pass
 
 
@@ -352,12 +374,12 @@ class Win32HtmlFormat:
 	'''This class adds support for Windows "HTML Format" clipboard content type
 
 	Code is based on example code from
-		http://code.activestate.com/recipes/474121/
+	U{http://code.activestate.com/recipes/474121/}
 
 	written by Phillip Piper (jppx1[at]bigfoot.com)
 
 	Also see specification at:
-		http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/winui/windowsuserinterface/dataexchange/clipboard/htmlclipboardformat.asp
+	U{http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/winui/windowsuserinterface/dataexchange/clipboard/htmlclipboardformat.asp}
 	'''
 
 	MARKER_BLOCK_OUTPUT = \
@@ -387,7 +409,7 @@ class Win32HtmlFormat:
 		#~ "EndHTML:(\d+)\s+" \
 		#~ "StartFragment:(\d+)\s+" \
 		#~ "EndFragment:(\d+)\s+" \
-		   #~ "SourceURL:(\S+)"
+		#~ "SourceURL:(\S+)"
 	#~ MARKER_BLOCK_RE = re.compile(MARKER_BLOCK)
 
 	DEFAULT_HTML_BODY = \
