@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 from __future__ import with_statement
 
@@ -14,13 +14,15 @@ import zim.datetimetz as datetime
 from zim.parsing import parse_date
 from zim.plugins import PluginClass
 from zim.notebook import Path
-from zim.gui.widgets import ui_environment, gtk_get_style,\
+from zim.gui.widgets import ui_environment, \
 	Dialog, MessageDialog, \
 	InputEntry, Button, IconButton, MenuButton, \
 	BrowserTreeView, SingleClickTreeView
+from zim.async import DelayedCallback
 from zim.formats import get_format, UNCHECKED_BOX, CHECKED_BOX, XCHECKED_BOX
 from zim.config import check_class_allow_empty
 
+from zim.plugins.calendar import daterange_from_path
 
 logger = logging.getLogger('zim.plugins.tasklist')
 
@@ -68,6 +70,9 @@ tag_re = re.compile(r'(?<!\S)@(\w+)\b', re.U)
 date_re = re.compile(r'\s*\[d:(.+)\]')
 
 
+_NO_DATE = '9999' # Constant for empty due date - value chosen for sorting properties
+
+
 # FUTURE: add an interface for this plugin in the WWW frontend
 
 # TODO allow more complex queries for filter, in particular (NOT tag AND tag)
@@ -99,15 +104,19 @@ This is a core plugin shipping with zim.
 			# T: label for plugin preferences dialog
 		('tag_by_page', 'bool', _('Turn page name into tags for task items'), False),
 			# T: label for plugin preferences dialog
+		('deadline_by_page', 'bool', _('Implicit deadline for task items in calendar pages'), False),
+			# T: label for plugin preferences dialog
 		('labels', 'string', _('Labels marking tasks'), 'FIXME, TODO', check_class_allow_empty),
 			# T: label for plugin preferences dialog - labels are e.g. "FIXME", "TODO", "TASKS"
 	)
-	_rebuild_on_preferences = ['all_checkboxes', 'labels']
+	_rebuild_on_preferences = ['all_checkboxes', 'labels','deadline_by_page']
 		# Rebuild database table if any of these preferences changed.
 		# But leave it alone if others change.
 
 	def __init__(self, ui):
 		PluginClass.__init__(self, ui)
+		self.task_labels = None
+		self.task_label_re = None
 		self.db_initialized = False
 
 	def initialize_ui(self, ui):
@@ -149,7 +158,7 @@ This is a core plugin shipping with zim.
 			self.task_labels = [s.strip() for s in self.preferences['labels'].split(',')]
 		else:
 			self.task_labels = []
-		regex = '(' + '|'.join(map(re.escape, self.task_labels)) + ')'
+		regex = '^(' + '|'.join(map(re.escape, self.task_labels)) + ')\\b'
 		self.task_label_re = re.compile(regex)
 
 	def _serialize_rebuild_on_preferences(self):
@@ -198,7 +207,12 @@ This is a core plugin shipping with zim.
 			parsetree = parser.parse(text)
 
 		#~ print '!! Checking for tasks in', path
-		tasks = self.extract_tasks(parsetree)
+		dates = daterange_from_path(path)
+		if dates and self.preferences['deadline_by_page']:
+			deadline = dates[2]
+		else:
+			deadline = None
+		tasks = self.extract_tasks(parsetree, deadline)
 		if tasks:
 			tasksfound = True
 
@@ -214,10 +228,10 @@ This is a core plugin shipping with zim.
 		if tasksfound:
 			self.emit('tasklist-changed')
 
-	def extract_tasks(self, parsetree):
+	def extract_tasks(self, parsetree, deadline=None):
 		'''Extract all tasks from a parsetree.
 		Returns tuples for each tasks with following properties:
-			(open, actionable, prio, due, description)
+		C{(open, actionable, prio, due, description)}
 		'''
 		tasks = []
 
@@ -238,7 +252,7 @@ This is a core plugin shipping with zim.
 						globaltags = []
 						break
 				else:
-					# no break occured - all OK
+					# no break occurred - all OK
 					lines.pop(0)
 					istasklist = True
 
@@ -249,11 +263,11 @@ This is a core plugin shipping with zim.
 					if istasklist or self.preferences['all_checkboxes'] \
 					or (self.task_labels and self.task_label_re.match(item[2])):
 						open = item[0] == UNCHECKED_BOX
-						tasks.append(self._parse_task(item[2], level=item[1], open=open, tags=globaltags))
+						tasks.append(self._parse_task(item[2], level=item[1], open=open, tags=globaltags, deadline=deadline))
 				else:
 					# normal line
 					if self.task_labels and self.task_label_re.match(item):
-						tasks.append(self._parse_task(item, tags=globaltags))
+						tasks.append(self._parse_task(item, tags=globaltags, deadline=deadline))
 
 		return tasks
 
@@ -306,17 +320,17 @@ This is a core plugin shipping with zim.
 			text += child.tail or ''
 		return text
 
-	def _parse_task(self, text, level=0, open=True, tags=None):
+	def _parse_task(self, text, level=0, open=True, tags=None, deadline=None):
 		# TODO - determine if actionable or not
 		prio = text.count('!')
 
 		global date # FIXME
-		date = '9999' # For sorting order this is good empty value
+		date = _NO_DATE
 
 		def set_date(match):
 			global date
 			mydate = parse_date(match.group(0))
-			if mydate and date == '9999':
+			if mydate and date == _NO_DATE:
 				date = '%04i-%02i-%02i' % mydate # (y, m, d)
 				#~ return match.group(0) # TEST
 				return ''
@@ -330,6 +344,10 @@ This is a core plugin shipping with zim.
 					text += ' ' + tag
 
 		text = date_re.sub(set_date, text)
+
+		if deadline and date == _NO_DATE:
+			date = deadline
+
 		return (open, True, prio, date, text)
 			# (open, actionable, prio, due, description)
 
@@ -428,17 +446,11 @@ class TaskListDialog(Dialog):
 		# Filter input
 		hbox.pack_start(gtk.Label(_('Filter')+': '), False) # T: Input label
 		filter_entry = InputEntry()
+		filter_entry.set_icon_to_clear()
 		hbox.pack_start(filter_entry, False)
-		clear_button = IconButton('gtk-clear')
-		hbox.pack_start(clear_button, False)
-		filter_button = Button(_('_Filter'), 'gtk-find') # T: Button
-		hbox.pack_start(filter_button, False)
-		filter_entry.connect('activate',
+		filter_cb = DelayedCallback(500,
 			lambda o: self.task_list.set_filter(filter_entry.get_text()))
-		filter_button.connect('clicked',
-			lambda o: self.task_list.set_filter(filter_entry.get_text()))
-		clear_button.connect('clicked',
-			lambda o: (filter_entry.set_text(''), filter_entry.activate()))
+		filter_entry.connect('changed', filter_cb)
 
 		# Dropdown with options - TODO
 		#~ menu = gtk.Menu()
@@ -456,15 +468,16 @@ class TaskListDialog(Dialog):
 		self.statistics_label = gtk.Label()
 		hbox.pack_end(self.statistics_label, False)
 
-		def set_statistics(task_list):
-			total, stats = task_list.get_statistics()
+		def set_statistics(o):
+			total, stats = self.task_list.get_statistics()
 			text = ngettext('%i open item', '%i open items', total) % total
 				# T: Label for statistics in Task List, %i is the number of tasks
 			text += ' (' + '/'.join(map(str, stats)) + ')'
 			self.statistics_label.set_text(text)
 
 		set_statistics(self.task_list)
-		self.task_list.connect('changed', set_statistics)
+		self.plugin.connect('tasklist-changed', set_statistics)
+			# Make sure this is connected after the task list connected to same signal
 
 	def do_response(self, response):
 		self.uistate['hpane_pos'] = self.hpane.get_position()
@@ -477,39 +490,51 @@ class TagListTreeView(SingleClickTreeView):
 	only show tasks with that tag.
 	'''
 
+	_type_separator = 0
+	_type_label = 1
+	_type_tag = 2
+
 	def __init__(self, task_list):
-		model = gtk.ListStore(str, bool) # tag name, is seperator bool
+		model = gtk.ListStore(str, int, int, int) # tag name, number of tasks, type, weight
 		SingleClickTreeView.__init__(self, model)
 		self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 		self.task_list = task_list
 
-		cell_renderer = gtk.CellRendererText()
-		column = gtk.TreeViewColumn(_('Tags'), cell_renderer, text=0)
+		column = gtk.TreeViewColumn(_('Tags'))
 			# T: Column header for tag list in Task List dialog
 		self.append_column(column)
 
-		self.set_row_separator_func(lambda m, i: m[i][1])
-			# just returns the bool in the second column
+		cr1 = gtk.CellRendererText()
+		cr1.set_property('ellipsize', pango.ELLIPSIZE_END)
+		column.pack_start(cr1, True)
+		column.set_attributes(cr1, text=0, weight=3) # tag name, weight
+
+		cr2 = self.get_cell_renderer_number_of_items()
+		column.pack_start(cr2, False)
+		column.set_attributes(cr2, text=1) # number of tasks
+
+		self.set_row_separator_func(lambda m, i: m[i][2] == self._type_separator)
 
 		self.get_selection().connect('changed', self.on_selection_changed)
 
 		self.refresh(task_list)
-		task_list.connect('changed', self.refresh)
+		task_list.plugin.connect('tasklist-changed', lambda o: self.refresh(task_list))
+			# Make sure this is connected after the task list connected to same signal
 
 	def get_tags(self):
 		'''Returns current selected tags, or None for all tags'''
-		tags = self._get_selected()
-		for label in self.task_list.plugin.task_labels:
-			if label in tags:
-				tags.remove(label)
+		tags = []
+		for row in self._get_selected():
+			if row[2] == self._type_tag:
+				tags.append(row[0])
 		return tags or None
 
 	def get_labels(self):
 		'''Returns current selected labels'''
 		labels = []
-		for tag in self._get_selected():
-			if tag in self.task_list.plugin.task_labels:
-				labels.append(tag)
+		for row in self._get_selected():
+			if row[2] == self._type_label:
+				labels.append(row[0])
 		return labels or None
 
 	def _get_selected(self):
@@ -517,18 +542,26 @@ class TagListTreeView(SingleClickTreeView):
 		if not paths or (0,) in paths:
 			return []
 		else:
-			return [model[path][0] for path in paths]
+			return [model[path] for path in paths]
 
 	def refresh(self, task_list):
 		# FIXME make sure selection is not reset when refreshing
 		model = self.get_model()
 		model.clear()
-		model.append((_('All'), False)) # T: "tag" for showing all tasks
-		for label in task_list.plugin.task_labels:
-			model.append((label, False))
-		model.append(('', True)) # separator
-		for tag in sorted(self.task_list.get_tags()):
-			model.append((tag, False))
+
+		n_all = self.task_list.get_n_tasks()
+		model.append((_('All Tasks'), n_all, self._type_label, pango.WEIGHT_BOLD)) # T: "tag" for showing all tasks
+
+		labels = self.task_list.get_labels()
+		for label in self.task_list.plugin.task_labels: # explicitly keep sorting from preferences
+			if label in labels:
+				model.append((label, labels[label], self._type_label, pango.WEIGHT_BOLD))
+
+		model.append(('', 0, self._type_separator, 0)) # separator
+
+		tags = self.task_list.get_tags()
+		for tag in sorted(tags):
+			model.append((tag, tags[tag], self._type_tag, pango.WEIGHT_NORMAL))
 
 	def on_selection_changed(self, selection):
 		tags = self.get_tags()
@@ -537,21 +570,14 @@ class TagListTreeView(SingleClickTreeView):
 		self.task_list.set_label_filter(labels)
 
 
-style = gtk_get_style()
-NORMAL_COLOR = style.base[gtk.STATE_NORMAL]
-HIGH_COLOR = gtk.gdk.color_parse('#EF2929') # red (from Tango style guide)
-MEDIUM_COLOR = gtk.gdk.color_parse('#FCAF3E') # orange ("idem")
-ALERT_COLOR = gtk.gdk.color_parse('#FCE94F') # yellow ("idem")
+HIGH_COLOR = '#EF5151' # red (derived from Tango style guide - #EF2929)
+MEDIUM_COLOR = '#FCB956' # orange ("idem" - #FCAF3E)
+ALERT_COLOR = '#FCEB65' # yellow ("idem" - #FCE94F)
 # FIXME: should these be configurable ?
 
 
 class TaskListTreeView(BrowserTreeView):
 
-	# define signals we want to use - (closure type, return type and arg types)
-	__gsignals__ = {
-		'changed': (gobject.SIGNAL_RUN_LAST, None, ()),
-	}
-
 	VIS_COL = 0 # visible
 	PRIO_COL = 1
 	TASK_COL = 2
@@ -581,12 +607,13 @@ class TaskListTreeView(BrowserTreeView):
 			if prio >= 3: color = HIGH_COLOR
 			elif prio == 2: color = MEDIUM_COLOR
 			elif prio == 1: color = ALERT_COLOR
-			else: color = NORMAL_COLOR
-			cell.set_property('cell-background-gdk', color)
+			else: color = None
+			cell.set_property('cell-background', color)
 
 		cell_renderer = gtk.CellRendererText()
-		column = gtk.TreeViewColumn(_('Prio'), cell_renderer)
+		#~ column = gtk.TreeViewColumn(_('Prio'), cell_renderer)
 			# T: Column header Task List dialog
+		column = gtk.TreeViewColumn(' ! ', cell_renderer)
 		column.set_cell_data_func(cell_renderer, render_prio)
 		column.set_sort_column_id(self.PRIO_COL)
 		self.append_column(column)
@@ -600,19 +627,21 @@ class TaskListTreeView(BrowserTreeView):
 		column.set_sort_column_id(self.TASK_COL)
 		column.set_expand(True)
 		if ui_environment['platform'] == 'maemo':
-			column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-			column.set_fixed_width(250)
-			# FIXME probably should also limit the size of this
-			# column on other platforms ...
+			column.set_min_width(250) # don't let this column get too small
+		else:
+			column.set_min_width(300) # don't let this column get too small
 		self.append_column(column)
 
+		if gtk.gtk_version >= (2, 12, 0):
+			self.set_tooltip_column(self.TASK_COL)
+
 		# Rendering of the Date column
 		today    = str( datetime.date.today() )
 		tomorrow = str( datetime.date.today() + datetime.timedelta(days=1))
 		dayafter = str( datetime.date.today() + datetime.timedelta(days=2))
 		def render_date(col, cell, model, i):
 			date = model.get_value(i, self.DATE_COL)
-			if date == '9999':
+			if date == _NO_DATE:
 				cell.set_property('text', '')
 			else:
 				cell.set_property('text', date)
@@ -621,8 +650,8 @@ class TaskListTreeView(BrowserTreeView):
 			if date <= today: color = HIGH_COLOR
 			elif date == tomorrow: color = MEDIUM_COLOR
 			elif date == dayafter: color = ALERT_COLOR
-			else: color = NORMAL_COLOR
-			cell.set_property('cell-background-gdk', color)
+			else: color = None
+			cell.set_property('cell-background', color)
 
 		cell_renderer = gtk.CellRendererText()
 		column = gtk.TreeViewColumn(_('Date'), cell_renderer)
@@ -635,7 +664,6 @@ class TaskListTreeView(BrowserTreeView):
 		cell_renderer = gtk.CellRendererText()
 		column = gtk.TreeViewColumn(_('Page'), cell_renderer, text=self.PAGE_COL)
 				# T: Column header Task List dialog
-		column.set_resizable(True)
 		column.set_sort_column_id(self.PAGE_COL)
 		self.append_column(column)
 
@@ -643,6 +671,9 @@ class TaskListTreeView(BrowserTreeView):
 		self.refresh()
 		self.plugin.connect_object('tasklist-changed', self.__class__.refresh, self)
 
+		# HACK because we can not register ourselves :S
+		self.connect('row_activated', self.__class__.do_row_activated)
+
 	def refresh(self):
 		self.real_model.clear() # flush
 
@@ -662,8 +693,6 @@ class TaskListTreeView(BrowserTreeView):
 			modelrow[0] = self._filter_item(modelrow)
 			self.real_model.append(None, modelrow)
 
-		self.emit('changed')
-
 	def set_filter(self, string):
 		# TODO allow more complex queries here - same parse as for search
 		if string:
@@ -677,9 +706,36 @@ class TaskListTreeView(BrowserTreeView):
 			self.filter = None
 		self._eval_filter()
 
+	def get_labels(self):
+		'''Get all labels that are in use
+		@returns: a dict with labels as keys and the number of tasks
+		per label as value
+		'''
+		labels = {}
+		def collect(model, path, iter):
+			row = model[iter]
+			if not row[self.OPEN_COL]:
+				return # only count open items
+
+			desc = row[self.TASK_COL].decode('utf-8')
+			match = self.plugin.task_label_re.match(desc)
+			if match:
+				label = match.group(0)
+				if not label in labels:
+					labels[label] = 1
+				else:
+					labels[label] += 1
+
+		self.real_model.foreach(collect)
+
+		return labels
+
 	def get_tags(self):
-		'''Returns list of all tags that are in use for tasks'''
-		tags = set()
+		'''Get all tags that are in use
+		@returns: a dict with tags as keys and the number of tasks
+		per tag as value
+		'''
+		tags = {}
 
 		def collect(model, path, iter):
 			row = model[iter]
@@ -688,17 +744,35 @@ class TaskListTreeView(BrowserTreeView):
 
 			desc = row[self.TASK_COL].decode('utf-8')
 			for match in tag_re.findall(desc):
-				tags.add(match)
+				if not match in tags:
+					tags[match] = 1
+				else:
+					tags[match] += 1
 
 			if self.plugin.preferences['tag_by_page']:
 				name = row[self.PAGE_COL].decode('utf-8')
 				for part in name.split(':'):
-					tags.add(part)
+					if not part in tags:
+						tags[part] = 1
+					else:
+						tags[part] += 1
 
 		self.real_model.foreach(collect)
 
 		return tags
 
+	def get_n_tasks(self):
+		'''Get the number of tasks in the list
+		@returns: total number as a list
+		'''
+		counter = [0]
+		def count(model, path, iter):
+			if model[iter][self.OPEN_COL]:
+				# only count open items
+				counter[0] += 1
+		self.real_model.foreach(count)
+		return counter[0]
+
 	def get_statistics(self):
 		statsbyprio = {}
 
@@ -791,51 +865,111 @@ class TaskListTreeView(BrowserTreeView):
 		page = Path( model[path][self.PAGE_COL] )
 		task = unicode(model[path][self.TASK_COL])
 		self.ui.open_page(page)
-		self.ui.mainwindow.pageview.view.get_buffer().finder.find(task)
+		self.ui.mainwindow.pageview.find(task)
 
-	def do_button_release_event(self, event):
-		'''Handler for button-release-event, triggers popup menu'''
-		if event.button == 3:
-			self.emit('popup-menu')# FIXME do we need to pass x/y and button ?
-			return True
-		else:
-			return BrowserTreeView.do_button_release_event(self, event)
-
-	def do_popup_menu(self): # FIXME do we need to pass x/y and button ?
-		menu = gtk.Menu()
+	def do_initialize_popup(self, menu):
 		item = gtk.MenuItem(_("_Copy")) # T: menu item in context menu
 		item.connect_object('activate', self.__class__.copy_to_clipboard, self)
 		menu.append(item)
-		menu.show_all()
-		menu.popup(None, None, None, 3, 0)
-		return True
 
 	def copy_to_clipboard(self):
-		'''Exports currently visable elements from the tasks list'''
+		'''Exports currently visible elements from the tasks list'''
 		logger.debug('Exporting to clipboard current view of task list.')
+		text = self.get_visible_data_as_csv()
+		gtk.Clipboard().set_text(text.decode("UTF-8"))
+
+	def get_visible_data_as_csv(self):
+		text = ""
+		for prio, desc, date, page in self.get_visible_data():
+			prio = str(prio)
+			desc = '"' + desc.replace('"', '""') + '"'
+			text += ",".join((prio, desc, date, page)) + "\n"
+		return text
+
+	def get_visible_data_as_html(self):
+		html = '''\
+<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
+<html>
+	<head>
+		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
+		<title>Task List - Zim</title>
+		<meta name='Generator' content='Zim [%% zim.version %%]'>
+		<style type='text/css'>
+			table.tasklist {
+				border-width: 1px;
+				border-spacing: 2px;
+				border-style: solid;
+				border-color: gray;
+				border-collapse: collapse;
+			}
+			table.tasklist th {
+				border-width: 1px;
+				padding: 1px;
+				border-style: solid;
+				border-color: gray;
+			}
+			table.tasklist td {
+				border-width: 1px;
+				padding: 1px;
+				border-style: solid;
+				border-color: gray;
+			}
+			.high {background-color: %s}
+			.medium {background-color: %s}
+			.alert {background-color: %s}
+		</style>
+	</head>
+	<body>
+
+<h1>Task List - Zim</h1>
+
+<table class="tasklist">
+<tr><th>Prio</th><th>Task</th><th>Date</th><th>Page</th></tr>
+''' % (HIGH_COLOR, MEDIUM_COLOR, ALERT_COLOR)
+
+		today    = str( datetime.date.today() )
+		tomorrow = str( datetime.date.today() + datetime.timedelta(days=1))
+		dayafter = str( datetime.date.today() + datetime.timedelta(days=2))
+		for prio, desc, date, page in self.get_visible_data():
+			if prio >= 3: prio = '<td class="high">%s</td>' % prio
+			elif prio == 2: prio = '<td class="medium">%s</td>' % prio
+			elif prio == 1: prio = '<td class="alert">%s</td>' % prio
+			else: prio = '<td>%s</td>' % prio
+
+			if date and date <= today: date = '<td class="high">%s</td>' % date
+			elif date == tomorrow: date = '<td class="medium">%s</td>' % date
+			elif date == dayafter: date = '<td class="alert">%s</td>' % date
+			else: date = '<td>%s</td>' % date
+
+			desc = '<td>%s</td>' % desc
+			page = '<td>%s</td>' % page
+
+			html += '<tr>' + prio + desc + date + page + '</tr>'
+
+		html += '''\
+</table>
+
+	</body>
+
+</html>
+'''
+		return html
+
+	def get_visible_data(self):
+		rows = []
 		model = self.get_model()
-		tasks = ""
 		for row in model:
-			# only open items
-			if row[self.OPEN_COL]:
-				tags = set()
-				vis, prio, desc, due, path_name, actionable, opn = row
-				if due == "9999": due = "-"
-				for match in tag_re.findall(desc.decode("UTF-8")):
-					tags.add(match)
-					tasks += "Description: %s\nPriority: %s,  Actionable: %s,  Open: %s,  Due: %s\nPath: %s,  Tags: %s\n\n" % (desc, prio, actionable, opn, due, path_name, ", ".join(tags))
-		tasks += "Number of tasks: %s. Exported: %s" \
-				% (len(model), str(datetime.now()))
-		#~ print tasks
-		gtk.Clipboard().set_text(tasks.decode("UTF-8"))
+			prio = row[self.PRIO_COL]
+			desc = row[self.TASK_COL]
+			date = row[self.DATE_COL]
+			page = row[self.PAGE_COL]
+
+			if date == _NO_DATE:
+				date = ''
+
+			rows.append((prio, desc, date, page))
+		return rows
 
 # Need to register classes defining gobject signals
-gobject.type_register(TaskListTreeView)
-
-
-# TODO this plugin should be ported to using a table in the index database
-# needs to hook database init and page indexing
-# Needs to re-build the database when preferences changed
-# Needs statusbar or similar to notify when indexing still ongoing
-# Separating database class and Treemodel will also allow better separation
-# of data and interface code.
+#~ gobject.type_register(TaskListTreeView)
+# NOTE: enabling this line causes this treeview to have wrong theming under default ubuntu them !???
